import os
import sys
import datetime

class ManagedDBIsStale(Exception):
  def __init__(self, filename):
    self.__filename = filename

  def __str__(self):
    return 'Managed DB dump {} is stale. Cannot provide accurate information'.format(self.__filename)

class ManagedDB(object):
  __instance = None

  def __new__(self, filename = '/etc/mythic/backup-managed-db', accept_stale = False, accept_missing = False):
    if accept_stale:
      self.__instance = ManagedDB.__ManagedDB(filename, True, accept_missing)

    if self.__instance is None:
      self.__instance = ManagedDB.__ManagedDB(filename, False, accept_missing)
    return self.__instance

  class __ManagedDB(object):
    __MAX_IGNORE_DAYS = 3

    def __init__(self, filename, accept_stale, accept_missing):
      assert filename is not None
      assert isinstance(accept_stale, bool)
      assert isinstance(accept_missing, bool)
      self.__partitions = dict()
      self.__missing = accept_missing

      # Fatal if managed db is missing, unless we are happy to accept
      # We could raise an exception but we definitely do not want to
      # work, even if someone uses try: except: around us.
      #
      # Many of the tools expect a managed DB and not having one (or
      # an empty one) is bad
      if not os.path.isfile(filename) and not accept_missing: 
        print('==============================================')
        print('FATAL: Managed DB does not exist')
        print('==============================================')
        sys.exit(1)

      if not accept_missing:
        filename_stat = os.stat(filename)
        self.__age = datetime.datetime.fromtimestamp(filename_stat.st_mtime, tz = datetime.timezone.utc)
      else:
        self.__age = datetime.datetime.fromtimestamp(1)

      # Complain if we are using a stale version if accept_stale
      # is set, otherwise refuse to work
      if not accept_stale and not accept_missing:
        if not self.is_fresh():
          raise ManagedDBIsStale(filename)
      elif not accept_missing:
        print('==============================================')
        print('WARNING: Managed DB dump from billing is stale')
        print('Results from this check may be inaccurate')
        print('==============================================')
        print('')

      if not accept_missing:
        self.read_config(filename)
    
    def read_config(self, filename):
      if os.path.isfile(filename):
        self.__filename = filename
        with open(filename, 'r') as handle_config:
          for line in handle_config:
            parts = line.split()

            assert len(parts) == 7

            if int(parts[5]) == 1:
              active = True
            else:
              active = False

            self.__partitions[parts[0]] = {
              'name': parts[0],
              'account': int(parts[1]),
              'managed': int(parts[2]),
              'service': parts[3],
              'db_size': int(parts[4]),
              'active': active,
              'server': parts[6],
            }

          # Fatal if the file exists and we loaded 
          # nothing because we absolutely should have 
          # at least _one_ parsed line
          if len(self.__partitions) <= 0:
            print('==============================================')
            print('FATAL: Nothing was parsed from the Managed DB')
            print('Results based on this info would be dangerous')
            print('==============================================')
            sys.exit(1) 


    # Check if managed-db dump is fresh
    def is_fresh(self):
      if self.__missing:
        return False

      max_age = datetime.datetime.now(tz = datetime.timezone.utc) - datetime.timedelta(days = self.__MAX_IGNORE_DAYS)
      if self.__age < max_age:
        return False
      return True
 
    # Check if partition name is billed
    def is_billed(self, partition_name):
      if partition_name in self.__partitions:
        return True
      return False

    # Check if partition is managed
    def is_managed(self, partition_name):
      if self.is_billed(partition_name):
        if self.__partitions[partition_name]['managed'] > 0:
          return True
      return False

    # Check if partition is managed
    def is_enhanced(self, partition_name):
      if self.is_billed(partition_name):
        if self.__partitions[partition_name]['managed'] > 1:
          return True
      return False

    # Check if partition is active
    def is_active(self, partition_name):
      if self.__missing:
        return False

      if self.is_billed(partition_name):
        if self.__partitions[partition_name]['active'] > 0:
          return True
      return False

    # Get account for partition
    def get_account(self, partition_name):
      if self.is_billed(partition_name):
        return self.__partitions[partition_name]['account']
      else:
        return None

    # Get size of partition as billed
    def get_size(self, partition_name):
      if self.is_billed(partition_name):
        return self.__partitions[partition_name]['db_size']
      else:
        return -1

    def get_server(self, partition_name):
      if self.is_billed(partition_name):
        return self.__partitions[partition_name]['server']

    # Generator for partitions in billing
    def partitions(self):
      for partition in self.__partitions:
        yield partition
