import datetime
import re

from . import cmds
from . import parsing

def __parse_filesystems(lines, strip_prefix = False, add_prefix = False):
  filesystems = dict()
  regex = re.compile('^([a-z0-9\_\-:\.\/]+)\t([0-9]+)\t([0-9]+)\t([0-9]+)\t([0-9]+)\t([^\t]+)\t(yes|no)\t(on|noauto|off)\t(on|off)\t(on|off|-)', re.IGNORECASE)
  for line in lines.split('\n'):
    match = regex.search(line)
    if match is not None:

      name = match.group(1)
      refquota = int(match.group(2))
      available = int(match.group(3))
      used_dataset = int(match.group(4))
      used_snapshots = int(match.group(5))
      mountpoint = match.group(6)
      mounted = False
      canmount = True
      readonly = True
      zfs_snap_ignore = False

      if match.group(7) == 'yes':
        mounted = True

      if match.group(8) == 'off':
        canmount = False

      if match.group(9) == 'off':
        readonly = False

      if match.group(10) == 'on':
        zfs_snap_ignore = True

      if strip_prefix is not False:
        if not strip_prefix.endswith('/'):
          strip_prefix += '/'

        if name.startswith(strip_prefix):
          name = name[len(strip_prefix):]

      if add_prefix is not False:
        if not add_prefix.endswith('/'):
          add_prefix += '/'

        name = add_prefix + name

      # name,refquota,available,usedbydataset,usedbysnapshots

      filesystems[name] = {
        'name': name,
        'refquota': refquota,
        'available': available,
        'usedbydataset': used_dataset,
        'usedbysnapshots': used_snapshots,
        'mountpoint': mountpoint,
        'mounted': mounted,
        'canmount': canmount,
        'readonly': readonly,
        'zfs.snap:ignore': zfs_snap_ignore,
      }

  return filesystems

def __parse_snapshots(lines, strip_prefix = False, add_prefix = False):
  snapshots = dict()
  regex = re.compile('^([a-z0-9\_\-:\.\/]+)@(([a-z0-9]+)?-?([0-9]{4}-[0-9]{2}-[0-9]{2}T[a-z0-9\-:\/]+))', re.IGNORECASE)
  for line in lines.split('\n'):
    match = regex.search(line)
    if match is not None:
      filesystem = match.group(1)
      snapshot_type = None
      name = line.split('\t')[0]

      if strip_prefix is not False:
        if not strip_prefix.endswith('/'):
          strip_prefix += '/'

        if filesystem.startswith(strip_prefix):
          filesystem = name[len(strip_prefix):]

      if add_prefix is not False:
        if not add_prefix.endswith('/'):
          add_prefix += '/'

        filesystem = add_prefix + filesystem

      if filesystem not in snapshots:
        snapshots[filesystem] = []

      if match.group(3) is None:
        snapshot_type = 'specific'
      else:
        snapshot_type = match.group(3)

      time_string = strip_timestamp(match.group(2))

      snapshot = {
        'filesystem': filesystem,
        'type': snapshot_type,
        'time': parsing.date.parse_ISO8601(time_string, raise_on_err = True),
        'name': name,
      }

      snapshots[filesystem].append(snapshot)

  return snapshots

def __parse_pools(lines):
  regex = re.compile('^([a-z0-9\-:\/]+)\t+([a-z0-9\-:\/\.]+)\t+([0-9]+)', re.IGNORECASE)
  pools = dict()

  for line in lines.split('\n'):
    match = regex.search(line)

    if match is not None:
      if match.group(1) not in pools:
        pools[match.group(1)] = dict()

      pools.get(match.group(1))[match.group(2)] = int(match.group(3))

  return pools

def get_pools(host = None):

  if host is not None:
    (exit_code, zpool_list_output, zpool_list_err) = cmds.SSH(host, 'zpool get -pH name,size,free')
  else:
    (exit_code, zpool_list_output, zpool_list_err) = cmds.local('zpool get -pH name,size,free')

  return __parse_pools(str(zpool_list_output, 'utf-8'))

def get_snapshots(strip_prefix = False, add_prefix = False, host = None):
  zfs_list_cmd = 'zfs list -t snapshot -Ho name -s name'
  zfs_list_output = None
  zfs_list_err = None

  if host is not None:
    (exit_code, zfs_list_output, zfs_list_err) = cmds.SSH(host, zfs_list_cmd)
  else:
    (exit_code, zfs_list_output, zfs_list_err) = cmds.local(zfs_list_cmd)

  assert exit_code == 0, 'zfs list failed'

  return __parse_snapshots(str(zfs_list_output, 'utf-8'), strip_prefix, add_prefix)

def get_filesystems(strip_prefix = False, add_prefix = False, host = None, list_only = False):
  zfs_list_cmd = ('zfs list -t filesystem -pHo name,'
                  'refquota,available,usedbydataset,usedbysnapshots,'
                  'mountpoint,mounted,canmount,readonly,zfs.snap:ignore '
                  '-s name '
                  '-s refquota '
                  '-s available '
                  '-s usedbydataset '
                  '-s usedbysnapshots '
                  '-s mountpoint '
                  '-s mounted '
                  '-s canmount '
                  '-s readonly '
                  '-s zfs.snap:ignore')

  zfs_list_output = None
  zfs_list_err = None

  if host is not None:
    (exit_code, zfs_list_output, zfs_list_err) = cmds.SSH(host, zfs_list_cmd)
  else:
    (exit_code, zfs_list_output, zfs_list_err) = cmds.local(zfs_list_cmd)

  assert exit_code == 0, 'zfs list failed'

  if list_only:
    return list(__parse_filesystems(str(zfs_list_output, 'utf-8'), strip_prefix, add_prefix).keys())
  else:
    return __parse_filesystems(str(zfs_list_output, 'utf-8'), strip_prefix, add_prefix)

def strip_timestamp(snapshot):
  re_timestamp = re.compile('^([A-Za-z]{1,}-)?(.*)([Zz]{1}[+\-]?[0-9]{4})?$')
  match_timestamp = re_timestamp.match(snapshot)

  if match_timestamp is not None:
    if len(match_timestamp.groups()) > 2:
      return match_timestamp.group(2)
    else:
      return match_timestamp.group(1)


